import matplotlib.pyplot as plt
from jjy.framework.network import MultiLayerNet
import jjy.framework.layer as Layer
import random
from jjy.dataset.mnist import load_mnist
from jjy.framework.cuda import xp as np
import numpy as np2
import matplotlib.pyplot as plt



def filter_show(filters, nx=8, margin=3, scale=10, title=None):
    """
    c.f. https://gist.github.com/aidiary/07d530d5e08011832b12#file-draw_weight-py
    """
    # FN, C, FH, FW = filters.shape
    FN, FH, FW = filters.shape

    FN = min(FN, 32)
    ny = int(np.ceil(FN / nx))

    fig = plt.figure()
    fig.subplots_adjust(left=0, right=1, bottom=0, top=1, hspace=0.05, wspace=0.05)

    fig.suptitle(title)
    for i in range(FN):
        ax = fig.add_subplot(ny, nx, i + 1, xticks=[], yticks=[])
        ax.imshow(filters[i], cmap=plt.cm.viridis, interpolation='nearest')

    plt.show()


# (x_train, t_train), (x_test, t_test) = load_mnist(flatten=False)
load_data = np2.load("D:\\develop\\deep-learning-without-tensorflow\\src\\jjy\\example\\idol\\idol_images_gray_128_v2.npz", allow_pickle=True)
x_train = load_data["x_train"]


net = MultiLayerNet()
# net.load_model("D:\\develop\\deep-learning-without-tensorflow\\src\\jjy\\example\\train_weight_2021-06-06 202331.npz")
net.load_model("D:\\develop\\deep-learning-without-tensorflow\\src\\jjy\\example\\idol\\idol_result\\idol_train_weight_2021-06-06 090301_8907.npz")

img_idx = random.randrange(0, 10000)
target_img = x_train[img_idx]
print(target_img.shape)

x = np.array([target_img])

i = 0
for layer_name, layer in net.layers.items():
    # print(x.shape)
    if isinstance(layer, Layer.BatchNormalization) or isinstance(layer, Layer.Dropout):

        x = layer.forward(x, False)
    else:
        print(layer)
        x = layer.forward(x)
    if type(layer) == Layer.Convolution or type(layer) == Layer.Affine or type(layer)==Layer.Pooling:
        if len(np.asnumpy(x)[0].shape) >= 3:
            filter_show(np.asnumpy(x)[0], title=f"{layer_name}")
            i += 1

# for k, v in net.params.items():
#     print(k, v.shape)


# filter_show(np.asnumpy(net.params["W0"]))



import sys
import jjy.framework.initializer as Initializer
import jjy.framework.layer as Layer
import jjy.framework.optimizer as Optimizer
from jjy.framework.functions import *
# from functions import *
from jjy.framework.network import MultiLayerNet

from jjy.dataset.mnist import load_mnist
import matplotlib.pyplot as plt
import random


def main():
    (x_train, t_train), (x_test, t_test) = load_mnist(flatten=False)
    # print(x_train.shape, t_train.shape)
    # print(t_train[0])
    # net.load_model("train_weight_2021-06-06 200548.npz")

    net = MultiLayerNet(is_use_dropout=False)
    net.add_layer(Layer.Dense(256, input_size=(1, 28, 28), initializer=Initializer.He()))
    net.add_layer(Layer.Relu())
    net.add_layer(Layer.Dense(256, initializer=Initializer.He()))
    net.add_layer(Layer.Relu())
    net.add_layer(Layer.Dense(512, initializer=Initializer.He()))
    # net.add_layer(Layer.Conv2D(16, (3, 3), pad=1, input_size=(1, 28, 28), ),
    #               initializer=Initializer.He(),
    #               activation=Layer.Relu())
    # net.add_layer(Layer.BatchNormalization())
    # net.add_layer(Layer.Conv2D(32, (3, 3), pad=1, initializer=Initializer.He(), activation=Layer.Relu()))
    # net.add_layer(Layer.Pooling(pool_h=2, pool_w=2, stride=2))
    # net.add_layer(Layer.Dropout(0.2))
    # net.add_layer(Layer.Dense(10))
    net.add_layer(Layer.SoftmaxWithLoss())
    result = net.train(
        x_train, t_train, x_test, t_test, batch_size=100, iters_num=1000, print_epoch=1, evaluate_limit=100,
        is_use_progress_bar=True,
        optimizer=Optimizer.Adam(lr=0.001))

    # for k, v in net.params.items():
    # print(k, v.shape)

    # for i in range(5):
    #     img_idx = random.randrange(0, 10000)
    #
    #     plt.imshow(x_test[img_idx].reshape(28, 28), cmap='gray')
    #
    #     predict_num = np.argmax(net.predict(np.array([x_test[img_idx]]), train_flg=False), axis=1)[0]
    #     correct_num = t_test[img_idx]
    #
    #     plt.title(f"Predict : {predict_num}, Correct : {correct_num}",)
    #     plt.show()
    #     print(f"Predict : {predict_num}, Correct : {correct_num}")
    #
    # import pickle
    # import datetime
    # # Save pickle
    #
    # with open(f"train_data_{str(datetime.datetime.now())[:-7].replace(':', '')}.pickle", "wb") as fw:
    #     pickle.dump(result, fw)
    # net.save_model()

    print("============================================")


main()

print("done!")



import os
from PIL import Image
import jsonpickle as jsp
import matplotlib.pyplot as plt
from jjy.framework.cuda import xp as np
import numpy as np2

import cupy as cp

import time
import random
from tqdm import tqdm
import jjy.framework.initializer as Initializer
import jjy.framework.layer as Layer
import jjy.framework.optimizer as Optimizer
from jjy.framework.functions import *
# from functions import *
from jjy.framework.network import MultiLayerNet

from jjy.dataset.mnist import load_mnist
import datetime





net = MultiLayerNet()
net.load_model("./idol_result/idol_train_weight_2021-06-06 090301_8907.npz")

load_data = np2.load("./idol_images_gray_128_v2.npz", allow_pickle=True)

x_train = load_data["x_train"]
t_train = load_data["t_train"].astype(np2.int64)

x_test = load_data["x_test"]
t_test = load_data["t_test"].astype(np2.int64)


print(x_train.shape, t_train.shape, x_test.shape, t_test.shape)

x_test = x_test / 255

x_test2 = []
t_test2 = []



i = 0
for x, t in zip(x_test, t_test):
    i += 1
    # x_test2.append(x)
    # t_test2.append(t)

    predict_num = np.argmax(net.predict(cp.array([x]), train_flg=False), axis=1)[0]

    if t != predict_num:
        print(t, predict_num)
        transposed_array = np.transpose(x, (1, 2, 0))
        plt.imshow(transposed_array, cmap='gray')
        plt.show()
        print("Predict : ", ["IU", "Irene", "Arin"][int(predict_num)], "Real : ", ["IU", "Irene", "Arin"][int(t)])
        a = input()
        if a == "d":
            print("discard")
        else:
            x_test2.append(x)
            t_test2.append(t)
    else:
        x_test2.append(x)
        t_test2.append(t)

print(i)





# x_train = np2.array(x_train)
x_test = np2.array(x_test2)
# t_train = np2.array(t_train)
t_test = np2.array(t_test2)

x_test = x_test * 255

print(x_test.shape)
print(t_test.shape)
temp_data = {"x_train": x_train, "x_test": x_test, "t_train": t_train, "t_test": t_test}
np2.savez_compressed("./idol_images_gray_128_v4.npz", **temp_data)
